@@ -7,6 +7,7 @@ module ErrM where
 -- the Error monad: like Maybe type with error msgs
 
 import Control.Monad (MonadPlus(..), liftM)
+import Control.Applicative (Applicative(..), Alternative(..))
 
 data Err a = Ok a | Bad String
   deriving (Read, Show, Eq, Ord)
@@ -15,7 +16,13 @@ instance Monad Err where
   return      = Ok
   fail        = Bad
   Ok a  >>= f = f a
-  Bad s >>= f = Bad s
+  Bad s >>= _ = Bad s
+
+instance Applicative Err where
+  pure = Ok
+  (Bad s) <*> _ = Bad s
+  (Ok f) <*> o  = liftM f o
+
 
 instance Functor Err where
   fmap = liftM
@@ -24,3 +31,7 @@ instance MonadPlus Err where
   mzero = Bad "Err.mzero"
   mplus (Bad _) y = y
   mplus x       _ = x
+
+instance Alternative Err where
+  empty = mzero
+  (<|>) = mplus
